#include "tokenstype.h"

enum TokenClass {
    BUILT_IN_FUNC = 1,
    DATA_TYPE     = 2,
    KEYWORD_NAME  = 4
};


struct TokenInfo {
    std::string text;
    unsigned short attributes;
};



struct TokenTables {
    std::unordered_map<TokenType, TokenInfo> allTokensInfo;
    std::unordered_map<std::string, TokenType> keywords;

    void add(TokenType tokenType, const std::string& txt,
             unsigned short attribs = 0);

    TokenTables();
};


static TokenTables tokenTables;

void TokenTables::add (TokenType tokenType, const std::string& txt,
                        unsigned short attribs)
{
    allTokensInfo[tokenType] = TokenInfo{txt, attribs};
    if(attribs & KEYWORD_NAME)
        keywords[txt] = tokenType;
}

TokenTables::TokenTables() {

    add( TokenType::IDENTIFIER, "identifier" );
    add( TokenType::KEYWORD,    "keyword" );

    // Literals (bool is not here, it has two keywords: false and true)
    add( TokenType::NUMBER,     "number" );
    add( TokenType::STRING,     "string" );
    add( TokenType::COLOR,      "color" );

    // Parentheses of all kinds
    add( TokenType::LPAREN,     "(" );
    add( TokenType::RPAREN,     ")" );
    add( TokenType::LBRACKET,   "[" );
    add( TokenType::RBRACKET,   "]" );
    add( TokenType::LBRACE,     "{" );
    add( TokenType::RBRACE,     "}" );
    add( TokenType::LT,         "<" );
    add( TokenType::GT,         ">" );

    // assignments and resizes
    add( TokenType::EQUALS,     "=" );

    // miscellaneous
    add( TokenType::DOT,        "." );
    add( TokenType::COLON,      ":" );
    add( TokenType::SEMICOLON,  ";" );
    add( TokenType::COMMA,      "," );
    add( TokenType::WHITESPACE, "whitespace" );
    add( TokenType::COMMENT,    "comment" );


    // Built-in functions (they are also keywords)
    /*
     * TODO
    */
    // Miscelaneous
    add( TokenType::END_OF_FILE,   "eof" );
}



const std::string &tokenTypeToString(TokenType tokenType) {
    return tokenTables.allTokensInfo[tokenType].text;
}

bool isBuiltInFunc(TokenType tokenType)
{
    return tokenTables.allTokensInfo[tokenType].attributes & BUILT_IN_FUNC;
}

bool isKeyword(TokenType tokenType)
{
    return tokenTables.allTokensInfo[tokenType].attributes & KEYWORD_NAME;
}
